package com.CTF.server.controller;

import com.CTF.server.entity.*;
import com.CTF.server.repository.ArticleRepository;
import com.CTF.server.repository.MessageRepository;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/articles")
public class ArticleController {

    private final ArticleRepository articleRepository;
    private final MessageRepository messageRepository;

    public ArticleController(ArticleRepository articleRepository, MessageRepository messageRepository) {
        this.articleRepository = articleRepository;
        this.messageRepository = messageRepository;
    }

    @GetMapping
    public List<ArticleWithMessage> getAllArticles() {
        List<Article> articles = articleRepository.findAll();
        return articles.stream().map(article -> {
            List<Message> messages = messageRepository.findByArticleId(article.getId());
            ArticleWithMessage articleWithMessage = new ArticleWithMessage();
            articleWithMessage.setArticle(article);
            // set LowMessage list
            articleWithMessage.setMessages(messages.stream().map(message -> {
                LowMessage lowMessage = new LowMessage();
                lowMessage.setText(message.getText());
                lowMessage.setIp(message.getIp());
                lowMessage.setTime(message.getTime().toString());
                return lowMessage;
            }).toList());
            return articleWithMessage;
        }).toList();
    }

    @GetMapping("/{id}")
    public ResponseEntity<ArticleWithMessage> getArticleById(@PathVariable Long id) {
        Optional<Article> article = articleRepository.findById(id);

        if (article.isPresent()) {
            List<Message> messages = messageRepository.findByArticleId(id);
            ArticleWithMessage articleWithMessage = new ArticleWithMessage();
            articleWithMessage.setArticle(article.get());
            // set LowMessage list
            articleWithMessage.setMessages(messages.stream().map(message -> {
                LowMessage lowMessage = new LowMessage();
                lowMessage.setText(message.getText());
                lowMessage.setIp(message.getIp());
                lowMessage.setTime(message.getTime().toString());
                return lowMessage;
            }).toList());

            return ResponseEntity.ok(articleWithMessage);
        }

        return ResponseEntity.notFound().build();
    }

    @PostMapping
    public Article createArticle(@RequestBody Article article) {
        article.setDate(new Date());
        return articleRepository.save(article);
    }
}