package com.CTF.server.controller;

import com.CTF.server.entity.Maria;
import com.CTF.server.entity.Role;
import com.CTF.server.entity.User;
import com.CTF.server.repository.MariaRepository;
import com.CTF.server.repository.UserRepository;
import org.springframework.http.ResponseEntity;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@RestController
@RequestMapping("/api/public")
public class AuthController {

    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;
    private final MariaRepository mariaRepository;

    public AuthController(UserRepository userRepository, PasswordEncoder passwordEncoder, MariaRepository mariaRepository) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
        this.mariaRepository = mariaRepository;
    }

    @PostMapping("/register")
    public ResponseEntity<String> register(@RequestBody User user) {
        user.setPassword(passwordEncoder.encode(user.getPassword()));
        user.setRole(Role.USER);
        userRepository.save(user);
        return ResponseEntity.ok("User registered successfully!");
    }

    @PostMapping("/login")
    public ResponseEntity<String> login(@RequestBody User user) {
        Optional<User> optionalUser = userRepository.findByUsername(user.getUsername());
        if (optionalUser.isPresent() && passwordEncoder.matches(user.getPassword(), optionalUser.get().getPassword())) {
            if (optionalUser.get().getRole() == Role.ADMIN) {
                Optional<Maria> maria = mariaRepository.findById(1L);
                return ResponseEntity.ok(maria.get().getMaria());
            }
            return ResponseEntity.ok("AUthentifié");
        } else {
            return ResponseEntity.status(401).body("user");
        }
    }
}