package com.CTF.server.controller;

import com.CTF.server.entity.Article;
import com.CTF.server.entity.Message;
import com.CTF.server.repository.ArticleRepository;
import com.CTF.server.repository.MessageRepository;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/messages")
public class MessageController {

    private final MessageRepository messageRepository;
    private final ArticleRepository articleRepository;

    public MessageController(MessageRepository messageRepository, ArticleRepository articleRepository) {
        this.messageRepository = messageRepository;
        this.articleRepository = articleRepository;
    }

    @GetMapping("/{articleId}")
    public ResponseEntity<List<Message>> getMessagesByArticleId(@PathVariable Long articleId) {
        Optional<Article> articleOptional = articleRepository.findById(articleId);
        if (articleOptional.isPresent()) {
            List<Message> messages = messageRepository.findByArticleId(articleId);
            return ResponseEntity.ok(messages);
        }
        return ResponseEntity.notFound().build();
    }

    @PostMapping("")
    public ResponseEntity<Message> addMessage(@RequestBody Message message, @RequestHeader(value = "X-Forwarded-For", required = false) String clientIp) {
        Optional<Article> articleOptional = articleRepository.findById(message.getArticle().getId());
        if (articleOptional.isPresent()) {
            message.setArticle(articleOptional.get());
            message.setTime(new Date());

            // Si l'IP est fournie dans les en-têtes, sinon null
            message.setIp(clientIp);

            Message savedMessage = messageRepository.save(message);
            return new ResponseEntity<>(savedMessage, HttpStatus.CREATED);
        }
        return ResponseEntity.notFound().build();
    }

    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteMessage(@PathVariable Long id) {
        if (messageRepository.existsById(id)) {
            messageRepository.deleteById(id);
            return ResponseEntity.noContent().build();
        }
        return ResponseEntity.notFound().build();
    }
}